from functools import lru_cache
from pathlib import Path

import pytest
import urllib.parse

ada_url = pytest.importorskip("ada_url")
can_ada = pytest.importorskip("can_ada")


@lru_cache
def data() -> list[str]:
    current_file_dir = Path(__file__).parent
    with open(current_file_dir / "data" / "top100.txt", "r") as f:
        return f.readlines()


def urllib_parse():
    for line in data():
        urllib.parse.urlparse(line)


def ada_python_parse():
    for line in data():
        try:
            ada_url.URL(line)
        except ValueError:
            # There are a small number of URLs in the sample data that are
            # not valid WHATWG URLs.
            pass


def can_ada_parse():
    for line in data():
        try:
            can_ada.parse(line)
        except ValueError:
            # There are a small number of URLs in the sample data that are
            # not valid WHATWG URLs.
            pass


@pytest.mark.slow
def test_urllib_parse(benchmark):
    benchmark(urllib_parse)


@pytest.mark.slow
def test_ada_python_parse(benchmark):
    benchmark(ada_python_parse)


@pytest.mark.slow
def test_can_ada_parse(benchmark):
    benchmark(can_ada_parse)



import can_ada


def test_encode():
    assert can_ada.idna_encode('Bücher.example') == b'xn--bcher-kva.example'


def test_decode():
    assert can_ada.idna_decode(b'xn--bcher-kva.example') == 'bücher.example'



from setuptools import setup
from pybind11.setup_helpers import Pybind11Extension

__version__ = "1.2.0"

setup(
    name="can_ada",
    version=__version__,
    author="Tyler Kennedy",
    author_email="tk@tkte.ch",
    long_description_content_type="text/markdown",
    long_description=open("README.md").read(),
    url="https://github.com/tktech/can_ada",
    description="Ada is a fast spec-compliant url parser",
    license="MIT",
    ext_modules=[
        Pybind11Extension(
            "can_ada",
            sorted(["src/binding.cpp", "src/ada.cpp"]),
            define_macros=[("VERSION_INFO", __version__)],
            language='c++',
            cxx_std=17
        ),
    ],
    package_data={
        "can_ada": [
            "py.typed",
            "can_ada.pyi"
        ]
    },
    extras_require={
        "test": [
            "pytest",
            "pytest-benchmark",
            "ada_url",
        ]
    },
    zip_safe=False,
    python_requires=">=3.7",
)



import can_ada


def test_can_parse():
    assert can_ada.can_parse("/", "https://tkte.ch") is True
    assert can_ada.can_parse("/", "/tkte.ch") is False


def test_parse():
    """
    Ensure we can simply call parse on a valid URL without crashing.
    """
    can_ada.parse("https://tkte.ch")


def test_port():
    """
    Ensure we can get and set the port.
    """
    url = can_ada.parse("https://tkte.ch")
    assert url.has_port() is False
    url.port = "9090"
    assert url.port == "9090"
